@@ -294,6 +294,11 @@ public abstract class AbstractActionApp
         {
             WrapperManager.appearHung();
             
+        }
+        else if ( action.equals( "appear_orphan" ) )
+        {
+            WrapperManager.appearOrphan();
+            
         }
         else if ( action.equals( "ignore_events" ) )
         {
